/*-----------------------------------------------------------------------------*
// include
*-----------------------------------------------------------------------------*/
#include "harktool.h"




/*-----------------------------------------------------------------------------*
// extern
*-----------------------------------------------------------------------------*/
extern struct GLOBAL  g;





/*-----------------------------------------------------------------------------*
* くちTSP録音から伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfmouth()
{



    harktool_LogI("harktoolcli_calctfmouth start");




    if (strlen(g.argv.mic_file) == 0) {
        harktool_LogE("argument error　[--mic-list]");
        harktool_help("calctfmouth");
        harktool_Exit(EXIT_FAILURE);
    } else if (strlen(g.argv.tsp_file) == 0) {
        harktool_LogE("argument error　[--tsp-list]");
        harktool_help("calctfmouth");
        harktool_Exit(EXIT_FAILURE);

    } else if (strlen(g.argv.output) == 0) {
        harktool_LogE("argument error　[--output-file]");
        harktool_help("calctfmouth");
        harktool_Exit(EXIT_FAILURE);

    }




    /*--------------------------------------------------*/
    //伝達関数作成
    /*--------------------------------------------------*/
    harktoolcli_calctfmouthExec();




    /*--------------------------------------------------*/
    // end proc
    /*--------------------------------------------------*/
    harkio_Log_printf(harkio_Log_Info, "@ harktoolcli_calctfmouth normal end");


    harktool_Exit(EXIT_SUCCESS);

}//











/*-----------------------------------------------------------------------------*
* くちTSP録音から伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void  harktoolcli_calctfmouthExec()
{



    /*--------------------------------------------------*/
    /* 伝達関数タイプ(定位・分離両方)                   */
    /*--------------------------------------------------*/
    // g.argv.tf_type = LOC_SEP;



    /*--------------------------------------------------*/
    // micファイル(microphones.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.mic_xml = harkio_XML_newFromXML(g.argv.mic_file , "");



    /*--------------------------------------------------*/
    //srcファイル(source.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.tsp_xml = harkio_XML_newFromXML(g.argv.tsp_file , "");


    /*--------------------------------------------------*/
    // position type
    /*--------------------------------------------------*/
    g.PosType = TSP;


    /*--------------------------------------------------*/
    //mic数
    /*--------------------------------------------------*/
    g.Nmic = harkio_Positions_getSize(g.mic_xml->poses);


    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    g.Nsrc = harkio_Positions_getSize(g.tsp_xml->poses);


    /*--------------------------------------------------*/
    // Config取得
    /*--------------------------------------------------*/
    g.Nfft           = harkio_Config_getNfft(g.tsp_xml->cfg);
    g.TspLength      = harkio_Config_getTSPLength(g.tsp_xml->cfg);
    g.SamplingRate   = harkio_Config_getSamplingRate(g.tsp_xml->cfg);
    g.SignalMax      = harkio_Config_getSignalMax(g.tsp_xml->cfg);
    g.SyncAverage    = harkio_Config_getSynchronousAverage(g.tsp_xml->cfg);
    g.OrgTspPath     = harkio_Config_getTSPpath(g.tsp_xml->cfg);
    g.TspOffset      = harkio_Config_getTSPoffset(g.tsp_xml->cfg);
    g.PeakSearchFrom = harkio_Config_getPeakSearchFrom(g.tsp_xml->cfg);
    g.PeakSearchTo   = harkio_Config_getPeakSearchTo(g.tsp_xml->cfg);


    g.IRsize = g.Nsrc * g.Nmic * g.Nfft ;
    g.TFsize = g.Nsrc * g.Nmic * (g.Nfft/2 + 1) ;




    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/

    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
    harktool_new(TF_VECT);


    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    harktool_new(TMP_TF_VECT);


    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    harktool_new(SEP_TF_VECT);



    //伝達関数作成用TEMP領域
    harktool_new(BUFFER);

    //Mic position
    harktool_new(MIC_POS_VECT);

    //SRC position
    harktool_new(SRC_POS_VECT);

    //SORT
    harktool_new(SORT_POS_ID);


    // インパルス応答ファイルのPATH格納領域
    harktool_new(FLT_PATH);


    // TSP wavファイル名格納領域
    harktool_new(TSP_WAV_NAME);



    /*--------------------------------------------------*/
    //  FFT data set
    /*--------------------------------------------------*/
    setFftData();



    /*--------------------------------------------------*/
    /*  Create Directory                                */
    /*--------------------------------------------------*/
    harktool_createTmpPath() ;





    /*--------------------------------------------------*/
    // id を昇順に並べ替えてpositionをセット
    /*--------------------------------------------------*/
    //MIC ID 並べ替え
    harktool_setMicPosId();

    //mic positionセット + 極座標->直交座標変換処理
    harktool_setMicPos();


    // TSP ID 並べ替え
    harktool_setTspPosId();

    //tsp positionセット + 極座標->直交座標変換処理
    harktool_setSrcPos();



    /*--------------------------------------------------*/
    // ChannelList init
    /*--------------------------------------------------*/
    harktool_setChannelList();


    /*--------------------------------------------------*/
    /* くちTSPでIR変換                                  */
    /* poses->pos.pathをくちTSPにわたす                 */
    /*--------------------------------------------------*/
    char buf[10]         ;
    int tsp_id           ;
    harkio_Position* pos ;
    int src              ;
    int i                ;



    for (src=0 ; src<g.Nsrc ; src++) {
        tsp_id = g.TspPosId[src] ;

        //tsp_idのpositionを取得
        pos = harkio_Positions_getById(g.tsp_xml->poses , tsp_id);


        /*--------------------------------------------------*/
        //position毎にtempディレクトリ作成
        /*--------------------------------------------------*/
        char *str;
        char strPath[PATH_LEN];

        str = strrchr(pos->path ,'/');

        if (str!=NULL) {
            //wav名を格納
            sprintf(g.tmp.tsp_wav_name[src] , "%s" , str+1);

            // /tmp/harktool/日時/tsp/xxx.wav ディレクトリ作成
            memset(strPath  , '\0'   , sizeof(strPath));
            sprintf(strPath , "%s/%s" , g.tmp.tsp_path , str+1);

            if (harktool_CheckDir(strPath) == NON_DIR_TYPE) {
                harktool_createDir(strPath);
            }
        } else {
            char msg[MSG_LEN];
            sprintf(msg, "can not get path [%s] ", pos->path);
            harktool_LogE(msg);
            harktool_Exit(EXIT_FAILURE);
        }


        //path copy
        sprintf(g.tmp.flt_path[src] , "%s" ,  strPath);  //fltファイル格納先path


        /// outimpfilehead = "/tmp/harktool/xxx.wav/float"
        sprintf(g.tmp.outimpfilehead , "%s/%s"  ,  g.tmp.flt_path[src] , FILE_PREFIX);


        /*--------------------------------------------------*/
        /* channellist 設定                                 */
        /*--------------------------------------------------*/

        if (pos->numChannels != 0) {
            memset(g.ChannelList , '\0' , sizeof(g.ChannelList));

            for (i=0; i< pos->numChannels; i++) {
                sprintf(buf , "%d,", pos->channels[i]);
                strcat(g.ChannelList , buf);
            }

            g.ChannelList[ strlen(g.ChannelList)-1 ] = 0;  //最後のカンマを削除

        }

        /*--------------------------------------------------*/
        /* harktool_mouthTspFD実行                          */
        /*--------------------------------------------------*/
//        Print_calcSysImp(pos->path);


        harktool_mouthTspFD(src                     ,  //src number
                            pos->path               , //inFile:  Input wav file name [String]
                            g.tmp.outimpfilehead    , //outFile: Output flt filename prefix. The name will be [outDir outFile '_ch*.flt']  [String]
                            g.argv.start            , //s: start point for Transfer Function(TF) calculation in sample [Integer]
                            g.argv.end              , //e:  end point for TF calculation in sample [Integer]
                            g.argv.mch              , // mch :Channel index of the voice input. The first channel is 1ch not 0ch. [Integer]
                            g.argv.nch                  , // nch: ChannelSelector for micarray channels.(max channel)
                            g.argv.fltlen           , // Number of samples output as TF [Integer]
                            g.Nfft                  , // Number of FFT points [Integer]
                            g.argv.hop              , // Shift length for TF calculation [Integer]
                            g.argv.headmargin       , // Head margin of impulse responces in samples [Integer(Default:0)]
                            //startid               , // start id of flt files like _ch0.flt if 0 [Integer(Default:1)]
                            g.argv.enable_wn        , // Enables adding white noise to escape unstable zero signal [Bool]
                            g.argv.wn_level         , // Level of white noise. Set enable_wn=true if you want to use this. [Double]
                            g.argv.enable_normalize   // Enable normalizing output TF
                           );

    }//src_for_end






    /*--------------------------------------------------*/
    /* Normalize                                        */
    /*--------------------------------------------------*/
    if (g.argv.enable_normalize == true) {
        harktool_normalizeTF();
    }


    /*--------------------------------------------------*/
    //定位伝達関数の配列からharkio_Matrix型に変換する
    /*--------------------------------------------------*/
    harktool_setLoctfFromArray();



    /*--------------------------------------------------*/
    //定位伝達関数追加
    /*--------------------------------------------------*/
    harkio_TransferFunction_setLocTFs(g.TF, g.Loctf);



    /*--------------------------------------------------*/
    //分離伝達関数の配列からharkio_Matrix型に変換する
    /*--------------------------------------------------*/
    harktool_copyTFvectFromSepTFvect(); //SepTFvect->TFvectにcopy

    harktool_setSeptfFromArray();



    /*--------------------------------------------------*/
    //分離伝達関数追加
    /*--------------------------------------------------*/
    harkio_TransferFunction_setSepTFs(g.TF, g.Septf);




    /*--------------------------------------------------*/
    // 伝達関数をZipファイルに出力
    //
    /*--------------------------------------------------*/
    harktool_writeTF();


    /*--------------------------------------------------*/
    //
    /*--------------------------------------------------*/
    createReadMeToZipFile(g.argv.output);


}//main end








