#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <libharkio3.h>







void help(void)
{
    printf("                                                             \n");
    printf("                                                             \n");
    printf("[NAME:]                                                      \n");
    printf("  harktoolcli-mat2csv                                        \n");
    printf("                                                             \n");
    printf("[Description:]                                               \n");
    printf("  Convert csv file from mat file                             \n");
    printf("                                                             \n");
    printf("[Usage :]                                                    \n");
    printf(" harktoolcli-mat2csv filename(mat file) -o output-filename]  \n");
    printf("                                                             \n");
    printf("[options]                                                    \n");
    printf("  -o : output file        (-o output.csv)                    \n");
    printf("                                                             \n");
    printf("                                                             \n");
}








int main(int argc, char **argv)
{

    int    option;
    int    matrix_type;
    char   infile[1024] = "";
    char   outfile[1024] = "";
    FILE   *fp = NULL;
    COMPLEX_TYPE  c_data;
    int    row;
    int    col;
    harkio_Matrix* mat;


    if (argc <= 1) {
    	harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    strncpy(infile , argv[1], 1024);

    // parse option
    while ((option = getopt(argc, argv, "o:")) != -1) {
        switch (option) {
        case 'o':
            strcpy(outfile, optarg);
            break;

        case '?':
            help();
            exit(EXIT_FAILURE);
        }
    }


    // read mat file
    mat = harkio_Matrix_fromFile(infile);
    if (mat == NULL) {
        printf("input file open error!!\n");
        exit(EXIT_FAILURE);
    }


    if (strlen(outfile) != 0) {
        if ((fp = fopen(outfile, "w")) == NULL) {
            printf("output file open error!!\n");
            exit(EXIT_FAILURE);
        }
        printf(" output file : %s\n",outfile);
    } else {
        fp = stdout;
    }



    matrix_type = harkio_CheckType(mat->type);
    if (matrix_type < 0) {
        harkio_Log_printf(harkio_Log_Error, "invalid matrix_type");
        exit(EXIT_FAILURE);
    }


    for (row=0 ; row< mat->rows; row++) {
        for (col=0 ; col < mat->cols ; col++) {
            int idx = col + row * mat->cols;

            switch (matrix_type) {
            case harkio_DataType_INT32:
                fprintf(fp , "%d" , mat->data.intdata[idx]);
                break;

            case harkio_DataType_FLOAT32:
                fprintf(fp , "%f" , mat->data.floatdata[idx]);
                break;

            case harkio_DataType_COMPLEX:
                c_data = mat->data.complexdata[idx];
                fprintf(fp , "%f,%f" , crealf(c_data) , cimagf(c_data));
                break;
            }

            if (col == mat->cols - 1) {
                fprintf(fp , "\n");
            } else {
                fprintf(fp , ",");
            }
        }
    }

    if (strlen(outfile) != 0) {
        fclose(fp);
    }
    fprintf(stderr, "SUCCESS\n");


    return EXIT_SUCCESS;

}

