#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>
#include <math.h>




float calcZValue(int z_unit , float re , float im);
int writeCoordinateFromTF(char *outfile, int z_axis, harkio_TransferFunction* tf, int tf_type, float fs,int nfft);
int writeCoordinateInternal(const char* outfile, int z_axis, harkio_Matrix** matdata, int num_src, int num_mic, int num_x, float x_weight, float y_weight);

#define DISP_REAL   1
#define DISP_AMP    2
#define DISP_db     3
#define DISP_PHASE  4





void help(void)
{
    printf("                                                              \n");
    printf("                                                              \n");
    printf("[NAME:]                                                       \n");
    printf("  harktoolcli-zip2coord                                       \n");
    printf("                                                              \n");
    printf("[Description:]                                                \n");
    printf("  Coordinate from transfer function                           \n");
    printf("                                                              \n");
    printf("[Usage:]                                                      \n");
    printf("  harktoolcli-zip2coord [options]                             \n");
    printf("                                                              \n");
    printf("[options]                                                     \n");
    printf("      -t: Type of Transfer function  (-t LOC)      [string]   \n");
    printf("            LOC : Localization transfer function              \n");
    printf("            SEP : Separation   transfer function              \n");
    printf("      -z: z_axis type                              [string]   \n");
    printf("              ( amp:amplitude   db:decibel   phase:phase )    \n");
    printf("      -i: input file                               [string]   \n");
    printf("      -o: output file                              [string]   \n");
//    printf("      -s: Sampling frequency           (-s 16000)    [integer] \n");
//    printf("      -n: Number of fft                (-n 512)      [integer] \n");
    printf("                                                              \n");
    printf("                                                              \n");

}









int main(int argc, char **argv)
{
    int option;

    char infile[2048];
    char outfile[2048];

    int samplingrate ;
    int fft_len ;
    int z_axis = DISP_AMP;
    int tf_type=0;


    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    // parse option
    while ((option = getopt(argc, argv, "t:z:o:i:s:n:")) != -1) {
        switch (option) {
        case 't':
            if (strcmp(optarg , "LOC") == 0 ||  strcmp(optarg , "loc") == 0) {
                tf_type=0;
            } else if (strcmp(optarg , "SEP") == 0 || strcmp(optarg , "sep") == 0) {
                tf_type=1;
            } else {
                help();
                exit(EXIT_FAILURE);
            }

            break;

        case 'z':
            if (strcmp(optarg , "amp") == 0) {
                z_axis  = DISP_AMP;
            } else if (strcmp(optarg , "db") == 0) {
                z_axis  = DISP_db;
            } else if (strcmp(optarg , "phase") == 0) {
                z_axis  = DISP_PHASE;
            } else {
                help();
                exit(EXIT_FAILURE);
            }

            break;

        case 'i':
            memset(infile , 0 , sizeof(infile));
            strncpy(infile , optarg, strlen(optarg));
            break;

        case 'o':
            memset(outfile , 0 , sizeof(outfile));
            strncpy(outfile , optarg, strlen(optarg));
            break;

//        case 's':
//			samplingrate = atoi(optarg);
//	        break;
//
//        case 'n':
//			fft_len = atoi(optarg);
//	        break;


        case '?':
            help();
            exit(EXIT_FAILURE);
        }
    }




    harkio_TransferFunction *tf = harkio_TransferFunction_fromFile(infile, NULL);

    fft_len = tf->cfg->nfft;
    samplingrate = tf->cfg->samplingRate;

    fprintf(stderr, "nfft=%d samplingrate=%d\n",fft_len,samplingrate);

    int ret = writeCoordinateFromTF(outfile , z_axis, tf, tf_type, samplingrate, fft_len);
    // harkio_TransferFunction_delete(&tf);
    if (ret == EXIT_SUCCESS) {
        fprintf(stderr, "SUCCESS\n");
    }
    return ret;
}



/*---------------------------------------*/
// Z軸計算
/*---------------------------------------*/
float calcZValue(int z_unit , float re , float im)
{

    char msg[MSG_LEN];


    switch (z_unit) {
    case DISP_REAL:
        return re;
        break;

    case DISP_AMP:
        return sqrtf(re*re + im*im);
        break;

    case DISP_db:
        return log10(sqrtf(re*re + im*im)) * 10;
        break;

    case DISP_PHASE:
        return atan2f(im, re) * 180/M_PI;
        break;

    default:
        sprintf(msg,"z_unit error [%d]", z_unit);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
        break;
    }

    return -1;
}


/*------------------------------------------------------*/
//  座標変換
/*------------------------------------------------------*/
int writeCoordinateInternal(const char* outfile, int z_axis, harkio_Matrix** matdata, int num_src, int num_mic, int num_x, float x_weight, float y_weight)
{
    //printf("writeCoordinate start\n");

    FILE *fpw;
    float real;
    float imag;
    float z_data;
    char  msg[MSG_LEN];
    int   src;
    int   mic;
    int   x;


    if ((fpw = fopen(outfile  , "w")) == NULL) {
        sprintf(msg,"fopen error [%s]", outfile);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }

    //write header
    fprintf(fpw, "id,mic,x,y,z\n");

    for (src=0 ; src < num_src ; src++) {
        //x軸がHzの場合: 256個
        //y軸がsrcの場合: 72個(matファイル数)
        //z軸がAMPの場合:
        int cnt = 0;

        for (mic=0 ; mic < num_mic ; mic++) {
            for (x=0 ; x< num_x ; x++) {
                float x_data = x * x_weight ;   // TIME[s] or FREQ
                float y_data = src * y_weight ;

                //z axis
                real = crealf(matdata[src]->data.complexdata[cnt]);
                imag = cimagf(matdata[src]->data.complexdata[cnt]);

                z_data = calcZValue(z_axis , real  , imag);

                fprintf(fpw ,   "%d , %d , %f , %f , %f\n", src , mic , x_data , y_data , z_data);
                cnt++;
            }

        }//mic for
    }//src for


    fclose(fpw);

    return EXIT_SUCCESS;
}

int writeCoordinateFromTF(char *outfile, int z_axis, harkio_TransferFunction* tf, int tf_type, float fs, int nfft)
{
    float x_weight   = fs/nfft;
    float y_weight   = 0.2;
    int   x_num      = nfft/2 + 1;
    harkio_Matrix** matdata = NULL;

    if (tf_type == 0) {
        matdata = tf->loctfs;
    } else if (tf_type == 1) {
        matdata = tf->septfs;
    } else {
        harkio_Log_printf(harkio_Log_Error , "given tf_type");
        return EXIT_FAILURE;
    }

    if (matdata == NULL) {
        harkio_Log_printf(harkio_Log_Error , "no data");
        return EXIT_FAILURE;
    }

    return writeCoordinateInternal(outfile, z_axis, matdata, tf->poses->size, matdata[0]->rows, x_num, x_weight, y_weight);
}

